﻿using System.Data.Common;
using Microsoft.Data.Sqlite;

namespace AndroidFlexcelTestApp
{
   public sealed class SQLiteDataAdapter : DbDataAdapter
   {
      private static object _updatingEventPH = new object();
      private static object _updatedEventPH = new object();
      private bool disposed;
      private bool disposeSelect = true;

      public SQLiteDataAdapter()
      {
      }

      public SQLiteDataAdapter(SqliteCommand cmd)
      {
         SelectCommand = cmd;
         disposeSelect = false;
      }

      public SQLiteDataAdapter(string commandText, SqliteConnection connection)
      {
         SelectCommand = new SqliteCommand(commandText, connection);
      }

      public SQLiteDataAdapter(
          string commandText,
          string connectionString
          )
      {
         SqliteConnection cnn = new SqliteConnection(
             connectionString);

         SelectCommand = new SqliteCommand(commandText, cnn);
      }

      private void CheckDisposed()
      {
         if (disposed)
            throw new ObjectDisposedException(nameof(SQLiteDataAdapter));
      }

      protected override void Dispose(bool disposing)
      {
         try
         {
            if (!disposed)
            {
               if (disposing)
               {
                  if (disposeSelect && (SelectCommand != null))
                  {
                     SelectCommand.Dispose();
                     SelectCommand = null;
                  }

                  if (InsertCommand != null)
                  {
                     InsertCommand.Dispose();
                     InsertCommand = null;
                  }

                  if (UpdateCommand != null)
                  {
                     UpdateCommand.Dispose();
                     UpdateCommand = null;
                  }

                  if (DeleteCommand != null)
                  {
                     DeleteCommand.Dispose();
                     DeleteCommand = null;
                  }
               }
            }
         }
         finally
         {
            base.Dispose(disposing);
            disposed = true;
         }
      }

      public event EventHandler<RowUpdatingEventArgs> RowUpdating
      {
         add
         {
            CheckDisposed();
            EventHandler<RowUpdatingEventArgs> previous = (EventHandler<RowUpdatingEventArgs>)Events[_updatingEventPH];
            if ((previous != null) && (value.Target is DbCommandBuilder))
            {
               EventHandler<RowUpdatingEventArgs> handler = (EventHandler<RowUpdatingEventArgs>)FindBuilder(previous);
               if (handler != null)
               {
                  Events.RemoveHandler(_updatingEventPH, handler);
               }
            }

            Events.AddHandler(_updatingEventPH, value);
         }
         remove { CheckDisposed(); Events.RemoveHandler(_updatingEventPH, value); }
      }

      internal static Delegate FindBuilder(MulticastDelegate mcd)
      {
         if (mcd != null)
         {
            Delegate[] invocationList = mcd.GetInvocationList();
            for (int i = 0; i < invocationList.Length; i++)
            {
               if (invocationList[i].Target is DbCommandBuilder)
               {
                  return invocationList[i];
               }
            }
         }
         return null;
      }

      public event EventHandler<RowUpdatedEventArgs> RowUpdated
      {
         add { CheckDisposed(); Events.AddHandler(_updatedEventPH, value); }
         remove { CheckDisposed(); Events.RemoveHandler(_updatedEventPH, value); }
      }

      protected override void OnRowUpdating(RowUpdatingEventArgs value)
      {
         EventHandler<RowUpdatingEventArgs> handler = Events[_updatingEventPH] as EventHandler<RowUpdatingEventArgs>;

         if (handler != null)
            handler(this, value);
      }

      protected override void OnRowUpdated(RowUpdatedEventArgs value)
      {
         EventHandler<RowUpdatedEventArgs> handler = Events[_updatedEventPH] as EventHandler<RowUpdatedEventArgs>;

         if (handler != null)
            handler(this, value);
      }

      public new SqliteCommand SelectCommand
      {
         get { CheckDisposed(); return (SqliteCommand)base.SelectCommand; }
         set { CheckDisposed(); base.SelectCommand = value; }
      }

      public new SqliteCommand InsertCommand
      {
         get { CheckDisposed(); return (SqliteCommand)base.InsertCommand; }
         set { CheckDisposed(); base.InsertCommand = value; }
      }

      public new SqliteCommand UpdateCommand
      {
         get { CheckDisposed(); return (SqliteCommand)base.UpdateCommand; }
         set { CheckDisposed(); base.UpdateCommand = value; }
      }

      public new SqliteCommand DeleteCommand
      {
         get { CheckDisposed(); return (SqliteCommand)base.DeleteCommand; }
         set { CheckDisposed(); base.DeleteCommand = value; }
      }
   }
}
